@@ -84,6 +84,7 @@ void serialize(Archive &ar, amici::Model &u, const unsigned int version) {
     ar &u.nxtrue_rdata;
     ar &u.nx_solver;
     ar &u.nxtrue_solver;
+    ar &u.nx_solver_reinit;
     ar &u.ny;
     ar &u.nytrue;
     ar &u.nz;
@@ -167,12 +168,14 @@ void serialize(Archive &ar, amici::ReturnData &r, const unsigned int version) {
     ar &r.cpu_time;
     ar &r.cpu_timeB;
     ar &r.preeq_cpu_time;
+    ar &r.preeq_cpu_timeB;
     ar &r.preeq_status;
     ar &r.preeq_numsteps;
     ar &r.preeq_numlinsteps;
     ar &r.preeq_wrms;
     ar &r.preeq_t;
     ar &r.posteq_cpu_time;
+    ar &r.posteq_cpu_timeB;
     ar &r.posteq_status;
     ar &r.posteq_numsteps;
     ar &r.posteq_numlinsteps;
