@@ -7,6 +7,7 @@
 #include "amici/symbolic_functions.h"
 #include "amici/vector.h"
 
+#include <cmath>
 #include <functional>
 #include <memory>
 
@@ -118,6 +119,21 @@ class Solver {
     void setupB(int *which, realtype tf, Model *model, const AmiVector &xB0,
                 const AmiVector &dxB0, const AmiVector &xQB0) const;
 
+    /**
+     * @brief Initialises the ami memory for quadrature computation
+     * @param t0 initial timepoint
+     * @param model pointer to the model instance
+     * @param x0 initial states
+     * @param dx0 initial derivative states
+     * @param xB0 initial adjoint states
+     * @param dxB0 initial derivative adjoint states
+     * @param xQ0 initial quadrature vector
+     */
+
+    void setupSteadystate(const realtype t0, Model *model, const AmiVector &x0,
+                          const AmiVector &dx0, const AmiVector &xB0,
+                          const AmiVector &dxB0, const AmiVector &xQ0) const;
+
     /**
      * @brief Reinitializes state and respective sensitivities (if necessary) according
      * to changes in fixedParameters
@@ -177,6 +193,18 @@ class Solver {
      */
     void setSensitivityMethod(SensitivityMethod sensi_meth);
 
+    /**
+     * @brief Return current sensitivity method during preequilibration
+     * @return method enum
+     */
+    SensitivityMethod getSensitivityMethodPreequilibration() const;
+
+    /**
+     * @brief Set sensitivity method for preequilibration
+     * @param sensi_meth_preeq
+     */
+    void setSensitivityMethodPreequilibration(SensitivityMethod sensi_meth_preeq);
+
     /**
      * @brief Disable forward sensitivity integration (used in steady state sim)
      */
@@ -544,7 +572,7 @@ class Solver {
      * @param ism internal sensitivity method
      */
     void setInternalSensitivityMethod(InternalSensitivityMethod ism);
-    
+
     /**
      * @brief returns the ReturnData reporting mode
      * @return ReturnData reporting mode
@@ -563,9 +591,10 @@ class Solver {
      * @param x state
      * @param dx derivative state
      * @param sx state sensitivity
+     * @param xQ quadrature
      */
     void writeSolution(realtype *t, AmiVector &x, AmiVector &dx,
-                       AmiVectorArray &sx) const;
+                       AmiVectorArray &sx, AmiVector &xQ) const;
 
     /**
      * @brief write solution from forward simulation
@@ -623,6 +652,13 @@ class Solver {
      */
     const AmiVector &getAdjointQuadrature(int which, realtype t) const;
 
+    /**
+     * @brief Access quadrature solution at time t
+     * @param t time
+     * @return (interpolated) solution xQ
+     */
+    const AmiVector &getQuadrature(realtype t) const;
+
     /**
      * @brief Reinitializes the states in the solver after an event occurence
      *
@@ -703,7 +739,7 @@ class Solver {
      * @return xQB.getLength()
      */
     int nquad() const;
-    
+
     /**
      * @brief check if FSA is being computed
      * @return flag
@@ -712,7 +748,7 @@ class Solver {
         return getSensitivityOrder() >= SensitivityOrder::first &&
         getSensitivityMethod() == SensitivityMethod::forward;
     }
-    
+
     /**
      * @brief check if ASA is being computed
      * @return flag
@@ -721,12 +757,12 @@ class Solver {
         return getSensitivityOrder() >= SensitivityOrder::first &&
         getSensitivityMethod() == SensitivityMethod::adjoint;
     }
-    
+
     /**
      * @brief Resets vectors containing diagnosis information
      */
     void resetDiagnosis() const;
-    
+
     /**
      * @brief Stores diagnosis information from solver memory block for forward problem
      */
@@ -738,7 +774,7 @@ class Solver {
      * @param which identifier of the backwards problem
      */
     void storeDiagnosisB(int which) const;
-    
+
     /**
      * @brief Accessor ns
      * @return ns
@@ -746,7 +782,7 @@ class Solver {
     std::vector<int> const& getNumSteps() const {
         return ns;
     }
-    
+
     /**
      * @brief Accessor nsB
      * @return nsB
@@ -754,7 +790,7 @@ class Solver {
     std::vector<int> const& getNumStepsB() const {
         return nsB;
     }
-    
+
     /**
      * @brief Accessor nrhs
      * @return nrhs
@@ -762,7 +798,7 @@ class Solver {
     std::vector<int> const& getNumRhsEvals() const {
         return nrhs;
     }
-    
+
     /**
      * @brief Accessor nrhsB
      * @return nrhsB
@@ -770,7 +806,7 @@ class Solver {
     std::vector<int> const& getNumRhsEvalsB() const {
         return nrhsB;
     }
-    
+
     /**
      * @brief Accessor netf
      * @return netf
@@ -778,7 +814,7 @@ class Solver {
     std::vector<int> const& getNumErrTestFails() const {
         return netf;
     }
-    
+
     /**
      * @brief Accessor netfB
      * @return netfB
@@ -786,7 +822,7 @@ class Solver {
     std::vector<int> const& getNumErrTestFailsB() const {
         return netfB;
     }
-    
+
     /**
      * @brief Accessor nnlscf
      * @return nnlscf
@@ -794,7 +830,7 @@ class Solver {
     std::vector<int> const& getNumNonlinSolvConvFails() const {
         return nnlscf;
     }
-    
+
     /**
      * @brief Accessor nnlscfB
      * @return nnlscfB
@@ -802,7 +838,7 @@ class Solver {
     std::vector<int> const& getNumNonlinSolvConvFailsB() const {
         return nnlscfB;
     }
-    
+
     /**
      * @brief Accessor order
      * @return order
@@ -895,6 +931,14 @@ class Solver {
      */
     virtual void getQuadB(int which) const = 0;
 
+    /**
+     * @brief extracts the quadrature at the current timepoint from solver
+     * memory and writes it to the xQ member variable
+     *
+     * @param t timepoint for quadrature extraction
+     */
+    virtual void getQuad(realtype &t) const = 0;
+
     /**
      * @brief Initialises the states at the specified initial timepoint
      *
@@ -905,6 +949,16 @@ class Solver {
     virtual void init(realtype t0, const AmiVector &x0,
                       const AmiVector &dx0) const = 0;
 
+    /**
+     * @brief Initialises the states at the specified initial timepoint
+     *
+     * @param t0 initial timepoint
+     * @param x0 initial states
+     * @param dx0 initial derivative states
+     */
+    virtual void initSteadystate(realtype t0, const AmiVector &x0,
+                                 const AmiVector &dx0) const = 0;
+
     /**
      * @brief initialises the forward sensitivities
      * @param sx0 initial states semsitivities
@@ -993,6 +1047,11 @@ class Solver {
      */
     virtual void setJacTimesVecFnB(int which) const = 0;
 
+    /**
+     * @brief sets the sparse Jacobian function for backward steady state case
+     */
+    virtual void setSparseJacFn_ss() const = 0;
+
     /**
      * @brief Create specifies solver method and initializes solver memory for
      * the forward problem
@@ -1034,6 +1093,14 @@ class Solver {
      */
     virtual void setQuadErrConB(int which, bool flag) const = 0;
 
+    /**
+     * @brief Specifies whether error control is also enforced for the
+     * forward quadrature problem
+     *
+     * @param flag activation flag
+     */
+    virtual void setQuadErrCon(bool flag) const = 0;
+
     /**
      * @brief Attaches the error handler function (errMsgIdAndTxt)
      * to the solver
@@ -1158,11 +1225,25 @@ class Solver {
     virtual void getQuadDkyB(realtype t, int k, int which) const = 0;
 
     /**
-     * @brief initializes the adjoint problem
+     * @brief interpolates the (derivative of the) solution at the requested
+     * timepoint
      *
+     * @param t timepoint
+     * @param k derivative order
+     */
+    virtual void getQuadDky(realtype t, int k) const = 0;
+
+    /**
+     * @brief initializes the adjoint problem
      */
     virtual void adjInit() const = 0;
 
+    /**
+     * @brief initializes the quadratures
+     * @param xQ0 vector with initial values for xQ
+     */
+    virtual void quadInit(const AmiVector &xQ0) const = 0;
+
     /**
      * @brief Specifies solver method and initializes solver memory for the
      * backward problem
@@ -1193,6 +1274,15 @@ class Solver {
     virtual void quadSStolerancesB(int which, realtype reltolQB,
                                    realtype abstolQB) const = 0;
 
+    /**
+     * @brief sets relative and absolute tolerances for the quadrature problem
+     *
+     * @param reltolQB relative tolerances
+     * @param abstolQB absolute tolerances
+     */
+    virtual void quadSStolerances(realtype reltolQB,
+                                  realtype abstolQB) const = 0;
+
     /**
      * @brief reports the number of solver steps
      *
@@ -1339,6 +1429,12 @@ class Solver {
      */
     bool getQuadInitDoneB(int which) const;
 
+    /**
+     * @brief checks whether memory for quadratures has been allocated
+     * @return proxy for solverMemory->(cv|ida)_QuadMallocDone
+     */
+    bool getQuadInitDone() const;
+
     /**
      * @brief attaches a diagonal linear solver to the forward problem
      */
@@ -1398,6 +1494,12 @@ class Solver {
      */
     void applyQuadTolerancesASA(int which) const;
 
+    /**
+     * @brief updates quadrature solver tolerances according to the
+     * currently specified member variables
+     */
+    void applyQuadTolerances() const;
+
     /**
      * @brief updates all senstivivity solver tolerances according to the
      * currently specified member variables
@@ -1485,6 +1587,19 @@ class Solver {
      */
     void setQuadInitDoneB(int which) const;
 
+    /**
+     * @brief sets that memory for quadratures has been allocated
+     */
+    void setQuadInitDone() const;
+
+    /**
+     * @brief Sets sensitivity method (for simulation or preequilibration)
+     * @param sensi_meth new value for sensi_meth[_preeq]
+     * @param preequilibration flag indicating preequilibration or simulation
+     */
+    void checkSensitivityMethod(const SensitivityMethod sensi_meth,
+                                bool preequilibration) const;
+
     /** state (dimension: nx_solver) */
     mutable AmiVector x = AmiVector(0);
 
@@ -1510,8 +1625,11 @@ class Solver {
     /** adjoint quadrature interface variable (dimension: nJ x nplist) */
     mutable AmiVector xQB = AmiVector(0);
 
+    /** forward quadrature interface variable (dimension: nx_solver) */
+    mutable AmiVector xQ = AmiVector(0);
+
     /** integration time of the forward problem */
-    mutable realtype t;
+    mutable realtype t = std::nan("");
 
     /** flag to force reInitPostProcessF before next call to solve */
     mutable bool forceReInitPostProcessF = false;
@@ -1523,6 +1641,9 @@ class Solver {
     /** method for sensitivity computation */
     SensitivityMethod sensi_meth = SensitivityMethod::forward;
 
+    /** method for sensitivity computation in preequilibration */
+    SensitivityMethod sensi_meth_preeq = SensitivityMethod::forward;
+
     /** flag controlling stability limit detection */
     booleantype stldet = true;
 
@@ -1584,7 +1705,7 @@ class Solver {
 
     /** relative tolerances for steadystate computation */
     realtype ss_rtol_sensi = NAN;
-    
+
     RDataReporting rdata_mode = RDataReporting::full;
 
     /** CPU time, forward solve */
@@ -1608,6 +1729,9 @@ class Solver {
     /** flag indicating whether adjInit was called */
     mutable bool adjInitialized = false;
 
+    /** flag indicating whether (forward) quadInit was called */
+    mutable bool quadInitialized = false;
+
     /** vector of flags indicating whether binit was called for respective
      which */
     mutable std::vector<bool> initializedB{false};
@@ -1618,7 +1742,7 @@ class Solver {
 
     /** number of checkpoints in the forward problem */
     mutable int ncheckPtr = 0;
-    
+
     /** number of integration steps forward problem (dimension: nt) */
     mutable std::vector<int> ns;
 
