@@ -97,6 +97,10 @@ class CVodeSolver : public Solver {
 
     void getQuadB(int which) const override;
 
+    void getQuad(realtype &t) const override;
+
+    void getQuadDky(realtype t, int k) const override;
+
     void reInitPostProcessF(realtype tnext) const override;
 
     void reInitPostProcessB(realtype tnext) const override;
@@ -115,6 +119,8 @@ class CVodeSolver : public Solver {
 
     void setQuadErrConB(int which, bool flag) const override;
 
+    void setQuadErrCon(bool flag) const override;
+
     void setErrHandlerFn() const override;
 
     void setUserData(Model *model) const override;
@@ -138,6 +144,8 @@ class CVodeSolver : public Solver {
 
     void adjInit() const override;
 
+    void quadInit(const AmiVector &xQ0) const override;
+    
     void allocateSolverB(int *which) const override;
 
     void setSStolerancesB(int which, realtype relTolB,
@@ -146,6 +154,8 @@ class CVodeSolver : public Solver {
     void quadSStolerancesB(int which, realtype reltolQB,
                            realtype abstolQB) const override;
 
+    void quadSStolerances(realtype reltolQ, realtype abstolQ) const override;
+
     void setMaxNumStepsB(int which, long int mxstepsB) const override;
 
     void diag() const override;
@@ -174,8 +184,11 @@ class CVodeSolver : public Solver {
 
     friend bool operator==(const CVodeSolver &a, const CVodeSolver &b);
 
-    void init(realtype t0, const AmiVector &x0, const AmiVector &dx0)
-    const override;
+    void init(realtype t0, const AmiVector &x0,
+              const AmiVector &dx0) const override;
+
+    void initSteadystate(const realtype t0, const AmiVector &x0,
+                         const AmiVector &dx0) const override;
 
     void sensInit1(const AmiVectorArray &sx0, const AmiVectorArray &sdx0)
     const override;
@@ -202,6 +215,8 @@ class CVodeSolver : public Solver {
     void setBandJacFnB(int which) const override;
 
     void setJacTimesVecFnB(int which) const override;
+
+    void setSparseJacFn_ss() const override;
 };
 
 } // namespace amici
