@@ -52,6 +52,8 @@ class IDASolver : public Solver {
     void quadSStolerancesB(int which, realtype reltolQB,
                            realtype abstolQB) const override;
 
+    void quadSStolerances(realtype reltolQ, realtype abstolQ) const override;
+
     int solve(realtype tout, int itask) const override;
 
     int solveF(realtype tout, int itask,
@@ -75,6 +77,10 @@ class IDASolver : public Solver {
 
     void getQuadDkyB(realtype t, int k, int which) const override;
 
+    void getQuad(realtype &t) const override;
+
+    void getQuadDky(realtype t, int k) const override;
+
     void calcIC(realtype tout1) const override;
 
     void calcICB(int which, realtype tout1) const override;
@@ -111,6 +117,8 @@ class IDASolver : public Solver {
 
     void setQuadErrConB(int which, bool flag) const override;
 
+    void setQuadErrCon(bool flag) const override;
+
     void setErrHandlerFn() const override;
 
     void setUserData(Model *model) const override;
@@ -135,6 +143,8 @@ class IDASolver : public Solver {
 
     void adjInit() const override;
 
+    void quadInit(const AmiVector &xQ0) const override;
+
     void allocateSolverB(int *which) const override;
 
     void setMaxNumStepsB(int which,
@@ -166,6 +176,9 @@ class IDASolver : public Solver {
     void init(realtype t0, const AmiVector &x0,
               const AmiVector &dx0) const override;
 
+    void initSteadystate(const realtype t0, const AmiVector &x0,
+                         const AmiVector &dx0) const override;
+
     void sensInit1(const AmiVectorArray &sx0, const AmiVectorArray &sdx0) const override;
 
     void binit(int which, realtype tf,
@@ -190,6 +203,8 @@ class IDASolver : public Solver {
     void setBandJacFnB(int which) const override;
 
     void setJacTimesVecFnB(int which) const override;
+
+    void setSparseJacFn_ss() const override;
 };
 
 } // namespace amici
