@@ -145,6 +145,12 @@ class SUNMatrixWrapper {
      */
     void reset();
 
+    /**
+     * @brief multiply with a scalar (in-place)
+     * @param a scalar value to multiply matrix
+     */
+    void scale(realtype a);
+
     /**
      * @brief N_Vector interface for multiply
      * @param c output vector, may already contain values
@@ -198,14 +204,13 @@ class SUNMatrixWrapper {
      */
     void zero();
 
+  private:
+    void update_ptrs();
+
     /**
      * @brief CSC matrix to which all methods are applied
      */
     SUNMatrix matrix = nullptr;
-
-  private:
-    void update_ptrs();
-
     realtype *data_ptr = nullptr;
     sunindextype *indexptrs_ptr = nullptr;
     sunindextype *indexvals_ptr = nullptr;
