@@ -144,10 +144,11 @@ class ReturnDataView(SwigPtrView):
         'ssigmaz', 'sllh', 's2llh', 'J', 'xdot', 'status', 'llh',
         'chi2', 'res', 'sres', 'FIM', 'w',
         'preeq_wrms', 'preeq_t', 'preeq_numlinsteps', 'preeq_numsteps',
-        'preeq_cpu_time', 'posteq_wrms', 'posteq_t', 'posteq_numlinsteps',
-        'posteq_numsteps', 'posteq_cpu_time',
-        'numsteps', 'numrhsevals', 'numerrtestfails',
-        'numnonlinsolvconvfails', 'order', 'cpu_time',
+        'preeq_numstepsB', 'preeq_status', 'preeq_cpu_time', 
+        'preeq_cpu_timeB', 'posteq_wrms', 'posteq_t', 'posteq_numlinsteps', 
+        'posteq_numsteps', 'posteq_numstepsB', 'posteq_status',
+        'posteq_cpu_time', 'posteq_cpu_timeB', 'numsteps', 'numrhsevals',
+        'numerrtestfails', 'numnonlinsolvconvfails', 'order', 'cpu_time',
         'numstepsB', 'numrhsevalsB', 'numerrtestfailsB',
         'numnonlinsolvconvfailsB', 'cpu_timeB'
     ]
@@ -198,8 +199,10 @@ class ReturnDataView(SwigPtrView):
             'xdot': [rdata.nx_solver],
             'preeq_numlinsteps': [rdata.newton_maxsteps, 2],
             'preeq_numsteps': [1, 3],
+            'preeq_status': [1, 3],
             'posteq_numlinsteps': [rdata.newton_maxsteps, 2],
             'posteq_numsteps': [1, 3],
+            'posteq_status': [1, 3],
             'numsteps': [rdata.nt],
             'numrhsevals': [rdata.nt],
             'numerrtestfails': [rdata.nt],
