@@ -301,6 +301,13 @@ void Model_ODE::fdxdotdw_rowvals(sunindextype * /*indexvals*/) {
                        __func__); // not implemented
 }
 
+void Model_ODE::fJB(const realtype t, realtype /*cj*/, const AmiVector &x,
+                     const AmiVector & /*dx*/, const AmiVector &xB,
+                     const AmiVector & /*dxB*/, const AmiVector &xBdot,
+                     SUNMatrix JB) {
+    fJB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(), JB);
+}
+
 void Model_ODE::fJB(realtype t, N_Vector x, N_Vector xB, N_Vector /*xBdot*/,
                     SUNMatrix JB) {
     auto x_pos = computeX_pos(x);
@@ -311,6 +318,13 @@ void Model_ODE::fJB(realtype t, N_Vector x, N_Vector xB, N_Vector /*xBdot*/,
         state.h.data(), N_VGetArrayPointer(xB), w.data(), dwdx.data());
 }
 
+void Model_ODE::fJSparseB(const realtype t, realtype /*cj*/, const AmiVector &x,
+                          const AmiVector & /*dx*/, const AmiVector &xB,
+                          const AmiVector & /*dxB*/, const AmiVector &xBdot,
+                          SUNMatrix JB) {
+    fJSparseB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(), JB);
+}
+
 void Model_ODE::fJSparseB(realtype t, N_Vector x, N_Vector xB,
                           N_Vector /*xBdot*/, SUNMatrix JB) {
     auto x_pos = computeX_pos(x);
@@ -380,6 +394,39 @@ void Model_ODE::fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot) {
     }
 }
 
+void Model_ODE::fxBdot_ss(const realtype t, const AmiVector &xB,
+                          const AmiVector & /*dx*/, AmiVector &xBdot) {
+    fxBdot_ss(t, xB.getNVector(), xBdot.getNVector());
+}
+
+void Model_ODE::fxBdot_ss(realtype /*t*/, N_Vector xB, N_Vector xBdot) const {
+    /* Right hande side of the adjoint state for steady state computations.
+       J is fixed (as x remeins in steady state), so the RHS becomes simple. */
+    N_VConst(0.0, xBdot);
+    J.multiply(xBdot, xB);
+}
+
+void Model_ODE::fqBdot_ss(realtype /*t*/, N_Vector xB, N_Vector qBdot) const {
+    /* Quadratures when computing adjoints for steady state. The integrand is
+       just the adjoint state itself. */
+    N_VScale(1.0, xB, qBdot);
+}
+
+void Model_ODE::fJSparseB_ss(SUNMatrix JB) {
+    /* Just copy the model Jacobian */
+    SUNMatCopy(J.get(), JB);
+}
+
+void Model_ODE::writeSteadystateJB(const realtype t, realtype /*cj*/,
+                                   const AmiVector &x, const AmiVector & /*dx*/,
+                                   const AmiVector &xB, const AmiVector & /*dxB*/,
+                                   const AmiVector &xBdot) {
+    /* Get backward Jacobian */
+    fJSparseB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(), J.get());
+    /* Switch sign, as we integrate forward in time, not backward */
+    J.scale(-1);
+}
+
 void Model_ODE::fsxdot(const realtype t, const AmiVector &x,
                        const AmiVector & /*dx*/, const int ip,
                        const AmiVector &sx, const AmiVector & /*sdx*/,
