@@ -5,7 +5,6 @@
 #include "amici/model.h"
 #include "amici/rdata.h"
 
-#include <cmath>
 #include <cstdio>
 #include <cstring>
 #include <ctime>
@@ -20,9 +19,10 @@ Solver::Solver(AmiciApplication *app) : app(app)
 
 Solver::Solver(const Solver &other)
     : ism(other.ism), lmm(other.lmm), iter(other.iter),
-      interpType(other.interpType), maxsteps(other.maxsteps), t(nan("")),
-      sensi_meth(other.sensi_meth), stldet(other.stldet),
-      ordering(other.ordering), newton_maxsteps(other.newton_maxsteps),
+      interpType(other.interpType), maxsteps(other.maxsteps),
+      sensi_meth(other.sensi_meth), sensi_meth_preeq(other.sensi_meth_preeq),
+      stldet(other.stldet), ordering(other.ordering),
+      newton_maxsteps(other.newton_maxsteps),
       newton_maxlinsteps(other.newton_maxlinsteps),
       newton_damping_factor_mode(other.newton_damping_factor_mode),
       newton_damping_factor_lower_bound(other.newton_damping_factor_lower_bound),
@@ -102,19 +102,19 @@ void Solver::setup(const realtype t0, Model *model, const AmiVector &x0,
     initializeLinearSolver(model);
     initializeNonLinearSolver();
 
-    if (sensi >= SensitivityOrder::first && model->nx_solver > 0) {
+    if (sensi >= SensitivityOrder::first &&
+        sensi_meth > SensitivityMethod::none && model->nx_solver > 0) {
         auto plist = model->getParameterList();
         sensInit1(sx0, sdx0);
         if (sensi_meth == SensitivityMethod::forward && !plist.empty()) {
             /* Set sensitivity analysis optional inputs */
             auto par = model->getUnscaledParameters();
 
-            /* Activate sensitivity calculations */
+            /* Activate sensitivity calculations  and apply tolerances */
             initializeNonLinearSolverSens(model);
             setSensParams(par.data(), nullptr, plist.data());
-
             applyTolerancesFSA();
-        } else if (sensi_meth == SensitivityMethod::adjoint) {
+        } else {
             /* Allocate space for the adjoint computation */
             adjInit();
         }
@@ -158,6 +158,27 @@ void Solver::setupB(int *which, const realtype tf, Model *model,
     setStabLimDetB(*which, stldet);
 }
 
+void Solver::setupSteadystate(const realtype t0, Model *model, const AmiVector &x0,
+                              const AmiVector &dx0, const AmiVector &xB0,
+                              const AmiVector &dxB0, const AmiVector &xQ0) const {
+    /* Initialize CVodes/IDAs solver with steadystate RHS function */
+    initSteadystate(t0, x0, dx0);
+
+    /* Allocate space for forward quadratures */
+    quadInit(xQ0);
+
+    /* Apply tolerances */
+    applyQuadTolerances();
+
+    /* Check linear solver (works only with KLU atm) */
+    if (linsol != LinearSolver::KLU)
+        throw AmiException("Backward steady state computation via integration "
+            "is currently only implemented for KLU linear solver");
+    /* Set Jacobian function and intialize values */
+    setSparseJacFn_ss();
+    model->writeSteadystateJB(t0, 0, x0, dx0, xB0, dxB0, xB0);
+}
+
 void Solver::updateAndReinitStatesAndSensitivities(Model *model) {
     model->fx0_fixedParameters(x);
     reInit(t, x, dx);
@@ -183,8 +204,14 @@ void Solver::resetDiagnosis() const {
 }
 
 void Solver::storeDiagnosis() const {
-    if (!solverWasCalledF || !solverMemory)
+    if (!solverWasCalledF || !solverMemory) {
+        ns.push_back(0);
+        nrhs.push_back(0);
+        netf.push_back(0);
+        nnlscf.push_back(0);
+        order.push_back(0);
         return;
+    }
 
     long int lnumber;
     getNumSteps(solverMemory.get(), &lnumber);
@@ -205,8 +232,13 @@ void Solver::storeDiagnosis() const {
 }
 
 void Solver::storeDiagnosisB(const int which) const {
-    if (!solverWasCalledB || !solverMemoryB.at(which))
+    if (!solverWasCalledB || !solverMemoryB.at(which)) {
+        nsB.push_back(0);
+        nrhsB.push_back(0);
+        netfB.push_back(0);
+        nnlscfB.push_back(0);
         return;
+    }
 
     long int number;
     getNumSteps(solverMemoryB.at(which).get(), &number);
@@ -440,16 +472,16 @@ bool operator==(const Solver &a, const Solver &b) {
 
 void Solver::applyTolerances() const {
     if (!getInitDone())
-        throw AmiException(("Solver instance was not yet set up, the "
-                            "tolerances cannot be applied yet!"));
+        throw AmiException("Solver instance was not yet set up, the "
+                           "tolerances cannot be applied yet!");
 
     setSStolerances(this->rtol, this->atol);
 }
 
 void Solver::applyTolerancesFSA() const {
     if (!getInitDone())
-        throw AmiException(("Solver instance was not yet set up, the "
-                            "tolerances cannot be applied yet!"));
+        throw AmiException("Solver instance was not yet set up, the "
+                           "tolerances cannot be applied yet!");
 
     if (sensi < SensitivityOrder::first)
         return;
@@ -463,8 +495,8 @@ void Solver::applyTolerancesFSA() const {
 
 void Solver::applyTolerancesASA(const int which) const {
     if (!getAdjInitDone())
-        throw AmiException(("Adjoint solver instance was not yet set up, the "
-                            "tolerances cannot be applied yet!"));
+        throw AmiException("Adjoint solver instance was not yet set up, the "
+                           "tolerances cannot be applied yet!");
 
     if (sensi < SensitivityOrder::first)
         return;
@@ -475,8 +507,8 @@ void Solver::applyTolerancesASA(const int which) const {
 
 void Solver::applyQuadTolerancesASA(const int which) const {
     if (!getAdjInitDone())
-        throw AmiException(("Adjoint solver instance was not yet set up, the "
-                            "tolerances cannot be applied yet!"));
+        throw AmiException("Adjoint solver instance was not yet set up, the "
+                           "tolerances cannot be applied yet!");
 
     if (sensi < SensitivityOrder::first)
         return;
@@ -490,6 +522,23 @@ void Solver::applyQuadTolerancesASA(const int which) const {
     quadSStolerancesB(which, quad_rtol, quad_atol);
 }
 
+void Solver::applyQuadTolerances() const {
+    if (!getQuadInitDone())
+        throw AmiException("Quadratures were not intialized, the "
+                           "tolerances cannot be applied yet!");
+
+    if (sensi < SensitivityOrder::first)
+        return;
+
+    realtype quad_rtolF = isNaN(this->quad_rtol) ? rtol : this->quad_rtol;
+    realtype quad_atolF = isNaN(this->quad_atol) ? atol : this->quad_atol;
+
+    /* Enable Quadrature Error Control */
+    setQuadErrCon(!std::isinf(quad_atolF) && !std::isinf(quad_rtolF));
+
+    quadSStolerances(quad_rtolF, quad_atolF);
+}
+
 void Solver::applySensitivityTolerances() const {
     if (sensi < SensitivityOrder::first)
         return;
@@ -504,14 +553,26 @@ void Solver::applySensitivityTolerances() const {
 
 SensitivityMethod Solver::getSensitivityMethod() const { return sensi_meth; }
 
+SensitivityMethod Solver::getSensitivityMethodPreequilibration() const { return sensi_meth_preeq; }
+
 void Solver::setSensitivityMethod(const SensitivityMethod sensi_meth) {
+    checkSensitivityMethod(sensi_meth, false);
+    this->sensi_meth = sensi_meth;
+}
+
+void Solver::setSensitivityMethodPreequilibration(const SensitivityMethod sensi_meth_preeq) {
+    checkSensitivityMethod(sensi_meth_preeq, true);
+    this->sensi_meth_preeq = sensi_meth_preeq;
+}
+
+void Solver::checkSensitivityMethod(const SensitivityMethod sensi_meth,
+                                    bool preequilibration) const {
     if (rdata_mode == RDataReporting::residuals &&
         sensi_meth == SensitivityMethod::adjoint)
         throw AmiException("Adjoint Sensitivity Analysis is not compatible with"
                            " only reporting residuals!");
-    if (sensi_meth != this->sensi_meth)
+    if (!preequilibration && sensi_meth != this->sensi_meth)
         resetMutableMemory(nx(), nplist(), nquad());
-    this->sensi_meth = sensi_meth;
 }
 
 int Solver::getNewtonMaxSteps() const { return newton_maxsteps; }
@@ -914,6 +975,8 @@ bool Solver::getQuadInitDoneB(const int which) const {
            initializedQB.at(which);
 }
 
+bool Solver::getQuadInitDone() const { return quadInitialized; }
+
 void Solver::setInitDone() const { initialized = true; };
 
 void Solver::setSensInitDone() const { sensInitialized = true; }
@@ -934,6 +997,8 @@ void Solver::setQuadInitDoneB(const int which) const {
     initializedQB.at(which) = true;
 }
 
+void Solver::setQuadInitDone() const { quadInitialized = true; }
+
 void Solver::switchForwardSensisOff() const {
     sensToggleOff();
     setSensInitOff();
@@ -953,6 +1018,7 @@ void Solver::resetMutableMemory(const int nx, const int nplist,
     initialized = false;
     adjInitialized = false;
     sensInitialized = false;
+    quadInitialized = false;
     solverWasCalledF = false;
     solverWasCalledB = false;
 
@@ -964,6 +1030,7 @@ void Solver::resetMutableMemory(const int nx, const int nplist,
     xB = AmiVector(nx);
     dxB = AmiVector(nx);
     xQB = AmiVector(nquad);
+    xQ = AmiVector(nx);
 
     solverMemoryB.clear();
     initializedB.clear();
@@ -971,13 +1038,14 @@ void Solver::resetMutableMemory(const int nx, const int nplist,
 }
 
 void Solver::writeSolution(realtype *t, AmiVector &x, AmiVector &dx,
-                           AmiVectorArray &sx) const {
+                           AmiVectorArray &sx, AmiVector &xQ) const {
     *t = gett();
+    if (quadInitialized)
+        xQ.copy(getQuadrature(*t));
+    if (sensInitialized)
+        sx.copy(getStateSensitivity(*t));
     x.copy(getState(*t));
     dx.copy(getDerivativeState(*t));
-    if (sensInitialized) {
-        sx.copy(getStateSensitivity(*t));
-    }
 }
 
 void Solver::writeSolutionB(realtype *t, AmiVector &xB, AmiVector &dxB,
@@ -1067,11 +1135,27 @@ const AmiVector &Solver::getAdjointQuadrature(const int which,
     return xQB;
 }
 
+const AmiVector &Solver::getQuadrature(realtype t) const {
+    if (quadInitialized) {
+        if (solverWasCalledF) {
+            if (t == this->t) {
+                getQuad(t);
+                return xQ;
+            }
+            getQuadDky(t, 0);
+        }
+    } else {
+        xQ.reset();
+    }
+    return xQ;
+}
+
+
 realtype Solver::gett() const { return t; }
 
 void wrapErrHandlerFn(int error_code, const char *module,
                       const char *function, char *msg, void * eh_data) {
-#define BUF_SIZE 250
+    constexpr int BUF_SIZE = 250;
     char buffer[BUF_SIZE];
     char buffid[BUF_SIZE];
     snprintf(buffer, BUF_SIZE, "AMICI ERROR: in module %s in function %s : %s ", module,
