@@ -79,6 +79,14 @@ static int fxBdot(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
 static int fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot,
                   void *user_data);
 
+static int fxBdot_ss(realtype t, N_Vector xB, N_Vector xBdot, void *user_data);
+
+static int fqBdot_ss(realtype t, N_Vector xB, N_Vector qBdot, void *user_data);
+
+static int fJSparseB_ss(realtype t, N_Vector x, N_Vector xBdot,
+                        SUNMatrix JB, void *user_data, N_Vector tmp1,
+                        N_Vector tmp2, N_Vector tmp3);
+
 static int fsxdot(int Ns, realtype t, N_Vector x, N_Vector xdot, int ip,
                   N_Vector sx, N_Vector sxdot, void *user_data,
                   N_Vector tmp1, N_Vector tmp2);
@@ -103,6 +111,15 @@ void CVodeSolver::init(const realtype t0, const AmiVector &x0,
         throw CvodeException(status, "CVodeInit");
 }
 
+void CVodeSolver::initSteadystate(const realtype t0, const AmiVector &x0,
+                                  const AmiVector &dx0) const {
+    /* We need to set the steadystate rhs function. SUndials doesn't have this
+       in its public api, so we have to change it in the solver memory,
+       as re-calling init would unset solver settings. */
+    auto cv_mem = static_cast<CVodeMem>(solverMemory.get());
+    cv_mem->cv_f = fxBdot_ss;
+}
+
 void CVodeSolver::sensInit1(const AmiVectorArray &sx0,
                             const AmiVectorArray & /*sdx0*/) const {
     int status = CV_SUCCESS;
@@ -211,6 +228,12 @@ void CVodeSolver::setJacTimesVecFnB(int which) const {
         throw CvodeException(status, "CVodeSetJacTimesB");
 }
 
+void CVodeSolver::setSparseJacFn_ss() const {
+    int status = CVodeSetJacFn(solverMemory.get(), fJSparseB_ss);
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeSetJacFn");
+}
+
 Solver *CVodeSolver::clone() const { return new CVodeSolver(*this); }
 
 void CVodeSolver::allocateSolver() const {
@@ -246,6 +269,12 @@ void CVodeSolver::setQuadErrConB(const int which, const bool flag) const {
         throw CvodeException(status, "CVodeSetQuadErrConB");
 }
 
+void CVodeSolver::setQuadErrCon(const bool flag) const {
+    int status = CVodeSetQuadErrCon(solverMemory.get(), flag);
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeSetQuadErrCon");
+}
+
 void CVodeSolver::getRootInfo(int *rootsfound) const {
     int status = CVodeGetRootInfo(solverMemory.get(), rootsfound);
     if (status != CV_SUCCESS)
@@ -553,6 +582,12 @@ void CVodeSolver::getQuadB(int which) const {
         throw CvodeException(status, "CVodeGetQuadB");
 }
 
+void CVodeSolver::getQuad(realtype &t) const {
+    int status = CVodeGetQuad(solverMemory.get(), &t, xQ.getNVector());
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeGetQuad");
+}
+
 void CVodeSolver::getQuadDkyB(const realtype t, const int k, int which) const {
     int status =
         CVodeGetQuadDky(CVodeGetAdjCVodeBmem(solverMemory.get(), which), t, k,
@@ -561,6 +596,13 @@ void CVodeSolver::getQuadDkyB(const realtype t, const int k, int which) const {
         throw CvodeException(status, "CVodeGetQuadDkyB");
 }
 
+void CVodeSolver::getQuadDky(const realtype t, const int k) const {
+    int status =
+    CVodeGetQuadDky(solverMemory.get(), t, k, xQ.getNVector());
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeGetQuadDky");
+}
+
 void CVodeSolver::adjInit() const {
     int status;
     if (getAdjInitDone()) {
@@ -574,6 +616,19 @@ void CVodeSolver::adjInit() const {
         throw CvodeException(status, "CVodeAdjInit");
 }
 
+void CVodeSolver::quadInit(const AmiVector &xQ0) const {
+    int status;
+    xQ.copy(xQ0);
+    if (getQuadInitDone()) {
+        status = CVodeQuadReInit(solverMemory.get(), xQ0.getNVector());
+    } else {
+        status = CVodeQuadInit(solverMemory.get(), fqBdot_ss, xQ.getNVector());
+        setQuadInitDone();
+    }
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeQuadInit");
+}
+
 void CVodeSolver::allocateSolverB(int *which) const {
     if (!solverMemoryB.empty()) {
         *which = 0;
@@ -605,6 +660,14 @@ void CVodeSolver::quadSStolerancesB(const int which, const realtype reltolQB,
         throw CvodeException(status, "CVodeQuadSStolerancesB");
 }
 
+void CVodeSolver::quadSStolerances(const realtype reltolQB,
+                                   const realtype abstolQB) const {
+    int status =
+    CVodeQuadSStolerances(solverMemory.get(), reltolQB, abstolQB);
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeQuadSStolerances");
+}
+
 void CVodeSolver::getB(const int which) const {
     realtype tDummy = 0;
     int status = CVodeGetB(solverMemory.get(), which, &tDummy, xB.getNVector());
@@ -710,12 +773,14 @@ void CVodeSolver::setStopTime(const realtype tstop) const {
         throw CvodeException(status, "CVodeSetStopTime");
 }
 
+
 void CVodeSolver::turnOffRootFinding() const {
     int status = CVodeRootInit(solverMemory.get(), 0, nullptr);
     if (status != CV_SUCCESS)
         throw CvodeException(status, "CVodeRootInit");
 }
 
+
 const Model *CVodeSolver::getModel() const {
     if (!solverMemory)
         throw AmiException("Solver has not been allocated, information is not "
@@ -724,6 +789,7 @@ const Model *CVodeSolver::getModel() const {
     return static_cast<Model *>(cv_mem->cv_user_data);
 }
 
+
 /**
  * @brief Jacobian of xdot with respect to states x
  * @param N number of state variables
@@ -737,7 +803,7 @@ const Model *CVodeSolver::getModel() const {
  * @param tmp3 temporary storage vector
  * @return status flag indicating successful execution
  **/
-int fJ(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
+static int fJ(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
                     void *user_data, N_Vector /*tmp1*/, N_Vector /*tmp2*/,
                     N_Vector /*tmp3*/) {
     auto model = static_cast<Model_ODE *>(user_data);
@@ -745,6 +811,7 @@ int fJ(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
     return model->checkFinite(gsl::make_span(J), "Jacobian");
 }
 
+
 /**
  * @brief Jacobian of xBdot with respect to adjoint state xB
  * @param NeqBdot number of adjoint state variables
@@ -759,7 +826,7 @@ int fJ(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
  * @param tmp3B temporary storage vector
  * @return status flag indicating successful execution
  **/
-int fJB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
+static int fJB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
                      SUNMatrix JB, void *user_data, N_Vector /*tmp1B*/,
                      N_Vector /*tmp2B*/, N_Vector /*tmp3B*/) {
     auto model = static_cast<Model_ODE *>(user_data);
@@ -767,6 +834,7 @@ int fJB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
     return model->checkFinite(gsl::make_span(JB), "Jacobian");
 }
 
+
 /**
  * @brief J in sparse form (for sparse solvers from the SuiteSparse Package)
  * @param t timepoint
@@ -779,7 +847,7 @@ int fJB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
  * @param tmp3 temporary storage vector
  * @return status flag indicating successful execution
  */
-int fJSparse(realtype t, N_Vector x, N_Vector /*xdot*/,
+static int fJSparse(realtype t, N_Vector x, N_Vector /*xdot*/,
                           SUNMatrix J, void *user_data, N_Vector /*tmp1*/,
                           N_Vector /*tmp2*/, N_Vector /*tmp3*/) {
     auto model = static_cast<Model_ODE *>(user_data);
@@ -787,6 +855,7 @@ int fJSparse(realtype t, N_Vector x, N_Vector /*xdot*/,
     return model->checkFinite(gsl::make_span(J), "Jacobian");
 }
 
+
 /**
  * @brief JB in sparse form (for sparse solvers from the SuiteSparse Package)
  * @param t timepoint
@@ -800,7 +869,7 @@ int fJSparse(realtype t, N_Vector x, N_Vector /*xdot*/,
  * @param tmp3B temporary storage vector
  * @return status flag indicating successful execution
  */
-int fJSparseB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
+static int fJSparseB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
                            SUNMatrix JB, void *user_data, N_Vector /*tmp1B*/,
                            N_Vector /*tmp2B*/, N_Vector /*tmp3B*/) {
     auto model = static_cast<Model_ODE *>(user_data);
@@ -808,6 +877,7 @@ int fJSparseB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
     return model->checkFinite(gsl::make_span(JB), "Jacobian");
 }
 
+
 /**
  * @brief J in banded form (for banded solvers)
  * @param N number of states
@@ -823,11 +893,12 @@ int fJSparseB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
  * @param tmp3 temporary storage vector
  * @return status flag indicating successful execution
  */
-int fJBand(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
+static int fJBand(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
            void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3) {
     return fJ(t, x, xdot, J, user_data, tmp1, tmp2, tmp3);
 }
 
+
 /**
  * @brief JB in banded form (for banded solvers)
  * @param NeqBdot number of states
@@ -844,12 +915,13 @@ int fJBand(realtype t, N_Vector x, N_Vector xdot, SUNMatrix J,
  * @param tmp3B temporary storage vector
  * @return status flag indicating successful execution
  */
-int fJBandB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
+static int fJBandB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
                          SUNMatrix JB, void *user_data, N_Vector tmp1B,
                          N_Vector tmp2B, N_Vector tmp3B) {
     return fJB(t, x, xB, xBdot, JB, user_data, tmp1B, tmp2B, tmp3B);
 }
 
+
 /**
  * @brief Matrix vector product of J with a vector v (for iterative solvers)
  * @param t timepoint
@@ -862,13 +934,14 @@ int fJBandB(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
  * @param tmp temporary storage vector
  * @return status flag indicating successful execution
  **/
-int fJv(N_Vector v, N_Vector Jv, realtype t, N_Vector x,
+static int fJv(N_Vector v, N_Vector Jv, realtype t, N_Vector x,
         N_Vector /*xdot*/, void *user_data, N_Vector /*tmp*/) {
     auto model = static_cast<Model_ODE *>(user_data);
     model->fJv(v, Jv, t, x);
     return model->checkFinite(gsl::make_span(Jv), "Jacobian");
 }
 
+
 /**
  * @brief Matrix vector product of JB with a vector v (for iterative solvers)
  * @param t timepoint
@@ -882,7 +955,7 @@ int fJv(N_Vector v, N_Vector Jv, realtype t, N_Vector x,
  * @param tmpB temporary storage vector
  * @return status flag indicating successful execution
  **/
-int fJvB(N_Vector vB, N_Vector JvB, realtype t, N_Vector x,
+static int fJvB(N_Vector vB, N_Vector JvB, realtype t, N_Vector x,
          N_Vector xB, N_Vector /*xBdot*/, void *user_data,
          N_Vector /*tmpB*/) {
     auto model = static_cast<Model_ODE *>(user_data);
@@ -890,6 +963,7 @@ int fJvB(N_Vector vB, N_Vector JvB, realtype t, N_Vector x,
     return model->checkFinite(gsl::make_span(JvB), "Jacobian");
 }
 
+
 /**
  * @brief Event trigger function for events
  * @param t timepoint
@@ -898,7 +972,7 @@ int fJvB(N_Vector vB, N_Vector JvB, realtype t, N_Vector x,
  * @param user_data object with user input @type Model_ODE
  * @return status flag indicating successful execution
  */
-int froot(realtype t, N_Vector x, realtype *root,
+static int froot(realtype t, N_Vector x, realtype *root,
                        void *user_data) {
     auto model = static_cast<Model_ODE *>(user_data);
     model->froot(t, x, gsl::make_span<realtype>(root, model->ne));
@@ -906,6 +980,7 @@ int froot(realtype t, N_Vector x, realtype *root,
                               "root function");
 }
 
+
 /**
  * @brief residual function of the ODE
  * @param t timepoint
@@ -914,7 +989,7 @@ int froot(realtype t, N_Vector x, realtype *root,
  * @param user_data object with user input @type Model_ODE
  * @return status flag indicating successful execution
  */
-int fxdot(realtype t, N_Vector x, N_Vector xdot, void *user_data) {
+static int fxdot(realtype t, N_Vector x, N_Vector xdot, void *user_data) {
     auto model = static_cast<Model_ODE *>(user_data);
 
     if (t > 1e200 && !model->checkFinite(gsl::make_span(x), "fxdot")) {
@@ -929,6 +1004,7 @@ int fxdot(realtype t, N_Vector x, N_Vector xdot, void *user_data) {
     return model->checkFinite(gsl::make_span(xdot), "fxdot");
 }
 
+
 /**
  * @brief Right hand side of differential equation for adjoint state xB
  * @param t timepoint
@@ -938,13 +1014,14 @@ int fxdot(realtype t, N_Vector x, N_Vector xdot, void *user_data) {
  * @param user_data object with user input @type Model_ODE
  * @return status flag indicating successful execution
  */
-int fxBdot(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
+static int fxBdot(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
                         void *user_data) {
     auto model = static_cast<Model_ODE *>(user_data);
     model->fxBdot(t, x, xB, xBdot);
     return model->checkFinite(gsl::make_span(xBdot), "fxBdot");
 }
 
+
 /**
  * @brief Right hand side of integral equation for quadrature states qB
  * @param t timepoint
@@ -954,13 +1031,68 @@ int fxBdot(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot,
  * @param user_data pointer to temp data object
  * @return status flag indicating successful execution
  */
-int fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot,
+static int fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot,
                         void *user_data) {
     auto model = static_cast<Model_ODE *>(user_data);
     model->fqBdot(t, x, xB, qBdot);
     return model->checkFinite(gsl::make_span(qBdot), "qBdot");
 }
 
+
+/**
+ * @brief Right hand side of differential equation for adjoint state xB
+ * when simulating in steadystate mode
+ * @param t timepoint
+ * @param xB Vector with the adjoint states
+ * @param xBdot Vector with the adjoint right hand side
+ * @param user_data object with user input @type Model_ODE
+ * @return status flag indicating successful execution
+ */
+static int fxBdot_ss(realtype t, N_Vector xB, N_Vector xBdot,
+                     void *user_data) {
+    auto model = static_cast<Model_ODE *>(user_data);
+    model->fxBdot_ss(t, xB, xBdot);
+    return model->checkFinite(gsl::make_span(xBdot), "fxBdot_ss");
+}
+
+
+/**
+ * @brief Right hand side of integral equation for quadrature states qB
+ * when simulating in steadystate mode
+ * @param t timepoint
+ * @param xB Vector with the adjoint states
+ * @param qBdot Vector with the adjoint quadrature right hand side
+ * @param user_data pointer to temp data object
+ * @return status flag indicating successful execution
+ */
+static int fqBdot_ss(realtype t, N_Vector xB, N_Vector qBdot,
+                     void *user_data) {
+    auto model = static_cast<Model_ODE *>(user_data);
+    model->fqBdot_ss(t, xB, qBdot);
+    return model->checkFinite(gsl::make_span(qBdot), "qBdot_ss");
+}
+
+/**
+ * @brief JB in sparse form for steady state case
+ * @param t timepoint
+ * @param x Vector with the states
+ * @param xBdot Vector with the adjoint right hand side
+ * @param JB Matrix to which the Jacobian will be written
+ * @param user_data object with user input @type Model_ODE
+ * @param tmp1B temporary storage vector
+ * @param tmp2B temporary storage vector
+ * @param tmp3B temporary storage vector
+ * @return status flag indicating successful execution
+ */
+static int fJSparseB_ss(realtype /*t*/, N_Vector /*x*/, N_Vector xBdot,
+                        SUNMatrix JB, void *user_data, N_Vector /*tmp1*/,
+                        N_Vector /*tmp2*/, N_Vector /*tmp3*/) {
+    auto model = static_cast<Model_ODE *>(user_data);
+    model->fJSparseB_ss(JB);
+    return model->checkFinite(gsl::make_span(xBdot), "JSparseB_ss");
+}
+
+
 /**
  * @brief Right hand side of differential equation for state sensitivities sx
  * @param Ns number of parameters
@@ -976,7 +1108,7 @@ int fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot,
  * @param tmp3 temporary storage vector
  * @return status flag indicating successful execution
  */
-int fsxdot(int /*Ns*/, realtype t, N_Vector x, N_Vector /*xdot*/,
+static int fsxdot(int /*Ns*/, realtype t, N_Vector x, N_Vector /*xdot*/,
                         int ip, N_Vector sx, N_Vector sxdot, void *user_data,
                         N_Vector /*tmp1*/, N_Vector /*tmp2*/) {
     auto model = static_cast<Model_ODE *>(user_data);
@@ -984,6 +1116,7 @@ int fsxdot(int /*Ns*/, realtype t, N_Vector x, N_Vector /*xdot*/,
     return model->checkFinite(gsl::make_span(sxdot), "sxdot");
 }
 
+
 bool operator==(const CVodeSolver &a, const CVodeSolver &b) {
     return static_cast<Solver const &>(a) == static_cast<Solver const &>(b);
 }
